import 'dart:async';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:panic_widgets/calendar/controller.dart';
import 'package:panic_widgets/calendar/day_names.dart';
import 'package:panic_widgets/calendar/header.dart';
import 'package:panic_widgets/calendar/options.dart';
import 'package:panic_widgets/calendar/month.dart';

const _dayNames = ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'];
const _monthNames = [
  'Jan',
  'Feb',
  'Mar',
  'Apr',
  'May',
  'Jun',
  'Jul',
  'Aug',
  'Sep',
  'Okt',
  'Nov',
  'Dec',
];

Map<int, DayOfWeekOptions> _mapDaysOfWeek(List<String> dayNames) {
  return {
    for (int i = 0; i < 5; i++) i + 1: DayOfWeekOptions(name: dayNames[i]),
    DateTime.saturday: DayOfWeekOptions(
      name: dayNames[5],
      nameTextColor: Colors.grey,
      textColor: Colors.grey,
    ),
    DateTime.sunday: DayOfWeekOptions(
      name: dayNames[6],
      nameTextColor: Colors.red[900],
      textColor: Colors.red,
    ),
  };
}

class PanicCalendar extends StatelessWidget {
  final PanicCalendarController c;

  final Widget topDivider;
  final Widget bottomDivider;

  PanicCalendar({
    Key key,
    List<String> monthNames = _monthNames,
    List<String> dayNames = _dayNames,
    Map<int, DayOfWeekOptions> daysOfWeek = const {},
    Map<DateTime, DateOptions> dateOptions = const {},
    Widget Function(BuildContext context, DateTime day) tileEventBuilder,
    FutureOr<void> Function(DateTime from, DateTime to) monthSwitched,
    FutureOr<void> Function(DateTime day) daySelected,
    double scrollSensitivity = 0.01,
    this.topDivider = const Divider(),
    this.bottomDivider = const Divider(),
  })  : c = Get.put(
          PanicCalendarController(
            monthNames: {
              for (int i = 1; i <= 12; i++) i: monthNames[i - 1],
            },
            daysOfWeek: _mapDaysOfWeek(dayNames)..addAll(daysOfWeek),
            dateOptions: dateOptions,
            tileEventBuilder: tileEventBuilder,
            onMonthSwitched: monthSwitched,
            onDaySelected: daySelected,
            scrollSensitivity: scrollSensitivity,
          ),
        ),
        super(key: key);

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        PanicCalendarHeader(),
        if (topDivider != null) topDivider,
        PanicCalendarDayNames(),
        if (bottomDivider != null) bottomDivider,
        Expanded(
          child: PageView(
            controller: c.pageController,
            onPageChanged: (page) => c.headerMonthOffset.value = page - 1,
            children: [
              for (int i = 0; i < 3; i++) PanicCalendarMonth(i - 1),
            ],
          ),
        ),
      ],
    );
  }
}
