import 'dart:async';

import 'package:date_util/date_util.dart';
import 'package:flutter/material.dart';
import 'package:get/state_manager.dart';
import 'package:panic_widgets/calendar/options.dart';

class PanicCalendarController extends GetxController {
  final dateUtil = DateUtil();

  final selectedDay = DateTime.now().obs;
  final Rx<DateTime> firstDayOfcurrentMonth;

  final headerMonthOffset = 0.obs;

  final double scrollSensitivity;

  final Map<int, String> monthNames;
  final Map<int, DayOfWeekOptions> daysOfWeek;
  final Map<DateTime, DateOptions> dateOptions;

  final Widget Function(BuildContext context, DateTime day) tileEventBuilder;

  final FutureOr<void> Function(DateTime from, DateTime to) _onMonthSwitched;
  final FutureOr<void> Function(DateTime day) onDaySelected;

  final PageController pageController = PageController(initialPage: 1, keepPage: false);

  PanicCalendarController({
    @required this.monthNames,
    @required this.daysOfWeek,
    @required this.dateOptions,
    @required this.tileEventBuilder,
    @required FutureOr<void> Function(DateTime from, DateTime to) onMonthSwitched,
    @required this.onDaySelected,
    @required this.scrollSensitivity,
  })  : this._onMonthSwitched = onMonthSwitched,
        firstDayOfcurrentMonth = DateTime(DateTime.now().year, DateTime.now().month, 1).obs {
    pageController.addListener(_onPageScroll);

    try {
      switchMonth(0);
    } catch (_ignored) {}
  }

  int dayInMonth(int row, int column, DateTime firstDay) => ((row * 7) + (column + 1)) - (firstDay.weekday + 1) + 1;
  int neededRows(DateTime firstDayInMonth) {
    int dim = dateUtil.daysInMonth(firstDayInMonth.month, firstDayInMonth.year);
    dim += firstDayInMonth.weekday - 1;
    return (dim / 7).ceil();
  }

  Color mixColors(Color c1, Color c2, double p) {
    double ip = 1 - p;

    return Color.fromARGB(
      (c1.alpha * p + c2.alpha * ip).round(),
      (c1.red * p + c2.red * ip).round(),
      (c1.green * p + c2.green * ip).round(),
      (c1.blue * p + c2.blue * ip).round(),
    );
  }

  Color getTileColor(BuildContext context, int i, Color color, DateTime firstDayOfcurrentMonth) {
    if (color != null) {
      if (i == selectedDay.value.day &&
          firstDayOfcurrentMonth.month == selectedDay.value.month &&
          firstDayOfcurrentMonth.year == selectedDay.value.year)
        return mixColors(color, Theme.of(context).primaryColor, 0.5);
      else
        return color;
    }

    if (i == selectedDay.value.day &&
        firstDayOfcurrentMonth.month == selectedDay.value.month &&
        firstDayOfcurrentMonth.year == selectedDay.value.year) {
      return Theme.of(context).primaryColor;
    }

    DateTime now = DateTime.now();

    if (i == now.day && firstDayOfcurrentMonth.month == now.month && firstDayOfcurrentMonth.year == now.year) {
      return Theme.of(context).primaryColor.withOpacity(0.5);
    }

    return null;
  }

  DateTime getTileDate(int row, int column, int monthOffset) {
    final dt = DateTime(
      firstDayOfcurrentMonth.value.year,
      firstDayOfcurrentMonth.value.month + monthOffset,
      1,
    );
    return DateTime(dt.year, dt.month, dayInMonth(row, column, dt));
  }

  DateTime getFirstDayOfOffsetMonth(int monthOffset) {
    return DateTime(firstDayOfcurrentMonth.value.year, firstDayOfcurrentMonth.value.month + monthOffset, 1);
  }

  Future<void> onTileTap(DateTime dt) async {
    selectedDay.value = dt;

    try {
      var r = (onDaySelected ?? (d) {})(dt);
      if (r is Future) await r;
    } catch (_) {}
  }

  void _onPageScroll() {
    if (pageController.page >= 0.9 && pageController.page < 1.1) return;
    final offset = pageController.page - pageController.page.floor();
    if (offset < scrollSensitivity || offset > 1 - scrollSensitivity) {
      switchMonth(pageController.page.round() - 1);
    }
  }

  void switchMonth(int add) async {
    final dt = DateTime(
      firstDayOfcurrentMonth.value.year,
      firstDayOfcurrentMonth.value.month + add,
      1,
    );

    final dtEnd = DateTime(
      dt.year,
      dt.month,
      dateUtil.daysInMonth(dt.month, dt.year),
    );

    try {
      var r = (_onMonthSwitched ?? (f, t) {})(dt, dtEnd);
      if (r is Future) await r;
    } catch (_) {}

    Timer(Duration(milliseconds: 1), () {
      firstDayOfcurrentMonth.value = dt;
      if (pageController.hasClients) pageController.jumpToPage(1);
    });
  }
}
