import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:panic_widgets/calendar/controller.dart';

class PanicCalendarTile extends StatelessWidget {
  final PanicCalendarController c = Get.find();

  final int row;
  final int column;

  final DateTime dt;

  final Color color;
  final Color textColor;

  final DateTime firstDayOfcurrentMonth;
  final DateTime firstDayOfbuildingMonth;

  PanicCalendarTile({
    Key key,
    @required this.row,
    @required this.column,
    @required this.dt,
    @required this.color,
    @required this.textColor,
    @required this.firstDayOfbuildingMonth,
  })  : firstDayOfcurrentMonth = DateTime(dt.year, dt.month, 1),
        super(key: key);

  @override
  Widget build(BuildContext context) {
    int i = c.dayInMonth(row, column, firstDayOfcurrentMonth);

    Widget tileEvents = (c.tileEventBuilder ?? (c, d) => null)(
      context,
      DateTime(firstDayOfcurrentMonth.year, firstDayOfcurrentMonth.month, i),
    );

    return Obx(
      () => IndexedStack(
        index: (firstDayOfcurrentMonth != firstDayOfbuildingMonth) ||
                (row == 0 && column < firstDayOfcurrentMonth.weekday) ||
                (i > c.dateUtil.daysInMonth(firstDayOfcurrentMonth.month, firstDayOfcurrentMonth.year))
            ? 1
            : 0,
        children: [
          Padding(
            padding: const EdgeInsets.all(2.0),
            child: FlatButton(
              onPressed: () async => c.onTileTap(dt),
              color: c.getTileColor(context, i, color, firstDayOfcurrentMonth),
              child: Column(
                mainAxisAlignment: MainAxisAlignment.center,
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisSize: MainAxisSize.max,
                children: [
                  if (tileEvents != null)
                    Flexible(
                      flex: 1,
                      child: IgnorePointer(
                        ignoring: true,
                        child: Opacity(
                          opacity: 0.0,
                          child: tileEvents,
                        ),
                      ),
                    ),
                  Flexible(
                    fit: FlexFit.tight,
                    child: Center(
                      child: Text(
                        i.toString(),
                        style: textColor != null ? TextStyle(color: textColor) : null,
                      ),
                    ),
                  ),
                  if (tileEvents != null)
                    Flexible(
                      flex: 1,
                      child: tileEvents,
                    ),
                ],
              ),
              shape: CircleBorder(),
            ),
          ),
          Container(),
        ],
      ),
    );
  }
}
